@@ -15,16 +15,15 @@ import sys
 if len(sys.argv) < 3:
     raise TypeError('usage: gpioget.py <gpiochip> <offset1> <offset2> ...')
 
-chip = gpiod.Chip(sys.argv[1])
+with gpiod.Chip(sys.argv[1]) as chip:
+    offsets = []
+    for off in sys.argv[2:]:
+        offsets.append(int(off))
 
-offsets = []
-for off in sys.argv[2:]:
-    offsets.append(int(off))
+    lines = chip.get_lines(offsets)
+    lines.request(consumer=sys.argv[0], type=gpiod.LINE_REQ_DIR_IN)
+    vals = lines.get_values()
 
-lines = chip.get_lines(offsets)
-lines.request(consumer=sys.argv[0], type=gpiod.LINE_REQ_DIR_IN)
-vals = lines.get_values()
-
-for val in vals:
-    print(val, end=' ')
-print()
+    for val in vals:
+        print(val, end=' ')
+    print()
