@@ -25,21 +25,20 @@ def print_event(event):
 if len(sys.argv) < 3:
     raise TypeError('usage: gpiomon.py <gpiochip> <offset1> <offset2> ...')
 
-chip = gpiod.Chip(sys.argv[1])
+with gpiod.Chip(sys.argv[1]) as chip:
+    offsets = []
+    for off in sys.argv[2:]:
+        offsets.append(int(off))
 
-offsets = []
-for off in sys.argv[2:]:
-    offsets.append(int(off))
+    lines = chip.get_lines(offsets)
+    lines.request(consumer=sys.argv[0], type=gpiod.LINE_REQ_EV_BOTH_EDGES)
 
-lines = chip.get_lines(offsets)
-lines.request(consumer=sys.argv[0], type=gpiod.LINE_REQ_EV_BOTH_EDGES)
-
-try:
-    while True:
-        ev_lines = lines.event_wait(sec=1)
-        if ev_lines:
-            for line in ev_lines:
-                event = line.event_read()
-                print_event(event)
-except KeyboardInterrupt:
-    sys.exit(130)
+    try:
+        while True:
+            ev_lines = lines.event_wait(sec=1)
+            if ev_lines:
+                for line in ev_lines:
+                    event = line.event_read()
+                    print_event(event)
+    except KeyboardInterrupt:
+        sys.exit(130)
